@@ -1,6 +1,6 @@
 package HTML::Scrubber;
 
-# ABSTRACT: Perl extension for scrubbing/sanitizing html
+# ABSTRACT: Perl extension for scrubbing/sanitizing HTML
 
 
 use 5.008;    # enforce minimum perl version of 5.8
@@ -9,10 +9,11 @@ use warnings;
 use HTML::Parser 3.47 ();
 use HTML::Entities;
 use Scalar::Util ('weaken');
+use List::Util qw(any);
 
 our ( @_scrub, @_scrub_fh );
 
-our $VERSION = '0.14'; # VERSION
+our $VERSION = '0.18'; # VERSION
 our $AUTHORITY = 'cpan:NIGELM'; # AUTHORITY
 
 # my my my my, these here to prevent foolishness like
@@ -20,6 +21,7 @@ our $AUTHORITY = 'cpan:NIGELM'; # AUTHORITY
 (@_scrub)    = ( \&_scrub,    "self, event, tagname, attr, attrseq, text" );
 (@_scrub_fh) = ( \&_scrub_fh, "self, event, tagname, attr, attrseq, text" );
 
+
 sub new {
     my $package = shift;
     my $p       = HTML::Parser->new(
@@ -263,6 +265,12 @@ sub _scrub_str {
         }
     }
     elsif ( $e eq 'end' ) {
+
+        # empty tags list taken from
+        # https://developer.mozilla.org/en/docs/Glossary/empty_element
+        my @empty_tags = qw(area base br col embed hr img input link meta param source track wbr);
+        return "" if $text ne '' && any { $t eq $_ } @empty_tags;    # skip false closing empty tags
+
         my $place = 0;
         if ( exists $s->{_rules}->{$t} ) {
             $place = 1 if $s->{_rules}->{$t};
@@ -280,13 +288,17 @@ sub _scrub_str {
         }
     }
     elsif ( $e eq 'comment' ) {
-        $outstr .= $text if $s->{_comment};
+        if ( $s->{_comment} ) {
+
+            # only copy comments through if they are well formed...
+            $outstr .= $text if ( $text =~ m|^<!--.*-->$|ms );
+        }
     }
     elsif ( $e eq 'process' ) {
         $outstr .= $text if $s->{_process};
     }
     elsif ( $e eq 'text' or $e eq 'default' ) {
-        $text =~ s/</&lt;/g;    #https://rt.cpan.org/Ticket/Attachment/8716/10332/scrubber.patch
+        $text =~ s/</&lt;/g;    #https://rt.cpan.org/Public/Ticket/Attachment/83958/10332/scrubber.patch
         $text =~ s/>/&gt;/g;
 
         $outstr .= $text;
@@ -361,15 +373,17 @@ __END__
 
 =pod
 
-=for stopwords html cpan callback homepage
+=encoding UTF-8
 
 =head1 NAME
 
-HTML::Scrubber - Perl extension for scrubbing/sanitizing html
+HTML::Scrubber - Perl extension for scrubbing/sanitizing HTML
 
 =head1 VERSION
 
-version 0.14
+version 0.18
+
+=for stopwords html cpan callback homepage Perlbrew perltidy repository
 
 =head1 SYNOPSIS
 
@@ -401,20 +415,43 @@ version 0.14
 If you want to "scrub" or "sanitize" html input in a reliable and flexible
 fashion, then this module is for you.
 
-I wasn't satisfied with HTML::Sanitizer because it is based on
-HTML::TreeBuilder, so I thought I'd write something similar that works directly
-with HTML::Parser.
+I wasn't satisfied with L<HTML::Sanitizer> because it is based on
+L<HTML::TreeBuilder>, so I thought I'd write something similar that works
+directly with L<HTML::Parser>.
 
 =head1 METHODS
 
 First a note on documentation: just study the L<EXAMPLE|"EXAMPLE"> below. It's
-all the documentation you could need
+all the documentation you could need.
 
 Also, be sure to read all the comments as well as L<How does it work?|"How does
 it work?">.
 
 If you're new to perl, good luck to you.
 
+=head2 new
+
+    my $scrubber = HTML::Scrubber->new( allow => [ qw[ p b i u hr br ] ] );
+
+Build a new L<HTML::Scrubber>.  The arguments are the initial values for the
+following directives:-
+
+=over 4
+
+=item * default
+
+=item * allow
+
+=item * deny
+
+=item * rules
+
+=item * process
+
+=item * comment
+
+=back
+
 =head2 comment
 
     warn "comments are  ", $p->comment ? 'allowed' : 'not allowed';
@@ -431,9 +468,9 @@ If you're new to perl, good luck to you.
         if $p->script;      # off by default
     $p->script( 0 || 1 );
 
-B<**> Please note that this is implemented using HTML::Parser's ignore_elements
-function, so if C<script> is set to true, all script tags encountered will be
-validated like all other tags.
+B<**> Please note that this is implemented using L<HTML::Parser>'s
+C<ignore_elements> function, so if C<script> is set to true, all script tags
+encountered will be validated like all other tags.
 
 =head2 style
 
@@ -441,9 +478,9 @@ validated like all other tags.
         if $p->style;       # off by default
     $p->style( 0 || 1 );
 
-B<**> Please note that this is implemented using HTML::Parser's ignore_elements
-function, so if C<style> is set to true, all style tags encountered will be
-validated like all other tags.
+B<**> Please note that this is implemented using L<HTML::Parser>'s
+C<ignore_elements> function, so if C<style> is set to true, all style tags
+encountered will be validated like all other tags.
 
 =head2 allow
 
@@ -468,11 +505,11 @@ validated like all other tags.
         ...
     );
 
-Updates set of attribute rules. Each rule can be 1/0, regular expression or a
-callback. Values longer than 1 char are treated as regexps. Callback is called
-with the following arguments: this object, tag name, attribute name and
-attribute value, should return empty list to drop attribute, C<undef> to keep
-it without value or a new scalar value.
+Updates a set of attribute rules. Each rule can be 1/0, a regular expression or
+a callback. Values longer than 1 char are treated as regexps. The callback is
+called with the following arguments: the current object, tag name, attribute
+name, and attribute value; the callback should return an empty list to drop the
+attribute, C<undef> to keep it without a value, or a new scalar value.
 
 =head2 default
 
@@ -512,28 +549,29 @@ Takes tag, rule('_' || $tag), attrref.
 
 =for comment _scrub_str
 
-I<default> handler, used by both _scrub and _scrub_fh Moved all the common code
-(basically all of it) into a single routine for ease of maintenance
+I<default> handler, used by both C<_scrub> and C<_scrub_fh>. Moved all the
+common code (basically all of it) into a single routine for ease of
+maintenance.
 
 =for comment _scrub_fh
 
 I<default> handler, does the scrubbing if we're scrubbing out to a file. Now
-calls _scrub_str and pushes that out to a file.
+calls C<_scrub_str> and pushes that out to a file.
 
 =for comment _scrub
 
 I<default> handler, does the scrubbing if we're returning a giant string. Now
-calls _scrub_str and appends that to the output string.
+calls C<_scrub_str> and appends that to the output string.
 
 =head1 How does it work?
 
-When a tag is encountered, HTML::Scrubber allows/denies the tag using the
+When a tag is encountered, L<HTML::Scrubber> allows/denies the tag using the
 explicit rule if one exists.
 
 If no explicit rule exists, Scrubber applies the default rule.
 
-If an explicit rule exists, but it's a simple rule(1), the default attribute
-rule is applied.
+If an explicit rule exists, but it's a simple rule(1), then the default
+attribute rule is applied.
 
 =head2 EXAMPLE
 
@@ -660,7 +698,7 @@ rule is applied.
 
 =head2 FUN
 
-If you have Test::Inline (and you've installed HTML::Scrubber), try
+If you have L<Test::Inline> (and you've installed L<HTML::Scrubber>), try
 
     pod2test Scrubber.pm >scrubber.t
     perl scrubber.t
@@ -669,7 +707,7 @@ If you have Test::Inline (and you've installed HTML::Scrubber), try
 
 L<HTML::Parser>, L<Test::Inline>.
 
-The C<HTML::Sanitizer> module is no longer available on CPAN.
+The L<HTML::Sanitizer> module is no longer available on CPAN.
 
 =head1 VERSION REQUIREMENTS
 
@@ -693,30 +731,13 @@ environment with L<Dist::Zilla>, and if you're just getting started, there's
 some documentation on using Vagrant and Perlbrew
 L<here|http://mrcaron.github.io/2015/03/06/Perl-CPAN-Pull-Request.html>.
 
-There is now a C<.perltidyrc> and a <.tidyallrc> file within the respository
+There is now a C<.perltidyrc> and a C<.tidyallrc> file within the repository
 for the standard perltidy settings used - I will apply these before new
 releases.  Please do not let formatting prevent you from sending in patches etc
 - this can be sorted out as part of the release process.  Info on C<tidyall>
 can be found at
 L<https://metacpan.org/pod/distribution/Code-TidyAll/bin/tidyall>.
 
-=head1 INSTALLATION
-
-See perlmodinstall for information and options on installing Perl modules.
-
-=head1 BUGS AND LIMITATIONS
-
-You can make new bug reports, and view existing ones, through the
-web interface at L<http://rt.cpan.org/Public/Dist/Display.html?Name=HTML-Scrubber>.
-
-=head1 AVAILABILITY
-
-The project homepage is L<https://metacpan.org/release/HTML-Scrubber>.
-
-The latest version of this module is available from the Comprehensive Perl
-Archive Network (CPAN). Visit L<http://www.perl.com/CPAN/> to find a CPAN
-site near you, or see L<https://metacpan.org/module/HTML::Scrubber/>.
-
 =head1 AUTHORS
 
 =over 4
@@ -737,9 +758,160 @@ D. H. <podmaster@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Ruslan Zakirov, Nigel Metheringham, 2003-2004 D. H..
+This software is copyright (c) 2018 by Ruslan Zakirov, Nigel Metheringham, 2003-2004 D. H.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Perldoc
+
+You can find documentation for this module with the perldoc command.
+
+  perldoc HTML::Scrubber
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+MetaCPAN
+
+A modern, open-source CPAN search engine, useful to view POD in HTML format.
+
+L<https://metacpan.org/release/HTML-Scrubber>
+
+=item *
+
+Search CPAN
+
+The default CPAN search engine, useful to view POD in HTML format.
+
+L<http://search.cpan.org/dist/HTML-Scrubber>
+
+=item *
+
+RT: CPAN's Bug Tracker
+
+The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
+
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=HTML-Scrubber>
+
+=item *
+
+AnnoCPAN
+
+The AnnoCPAN is a website that allows community annotations of Perl module documentation.
+
+L<http://annocpan.org/dist/HTML-Scrubber>
+
+=item *
+
+CPAN Ratings
+
+The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
+
+L<http://cpanratings.perl.org/d/HTML-Scrubber>
+
+=item *
+
+CPANTS
+
+The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
+
+L<http://cpants.cpanauthors.org/dist/HTML-Scrubber>
+
+=item *
+
+CPAN Testers
+
+The CPAN Testers is a network of smoke testers who run automated tests on uploaded CPAN distributions.
+
+L<http://www.cpantesters.org/distro/H/HTML-Scrubber>
+
+=item *
+
+CPAN Testers Matrix
+
+The CPAN Testers Matrix is a website that provides a visual overview of the test results for a distribution on various Perls/platforms.
+
+L<http://matrix.cpantesters.org/?dist=HTML-Scrubber>
+
+=item *
+
+CPAN Testers Dependencies
+
+The CPAN Testers Dependencies is a website that shows a chart of the test results of all dependencies for a distribution.
+
+L<http://deps.cpantesters.org/?module=HTML::Scrubber>
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<bug-html-scrubber at rt.cpan.org>, or through
+the web interface at L<https://rt.cpan.org/Public/Bug/Report.html?Queue=HTML-Scrubber>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
+
+The code is open to the world, and available for you to hack on. Please feel free to browse it and play
+with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
+from your repository :)
+
+L<https://github.com/nigelm/html-scrubber>
+
+  git clone https://github.com/nigelm/html-scrubber.git
+
+=head1 CONTRIBUTORS
+
+=for stopwords Andrei Vereha Lee Johnson Michael Caron Nigel Metheringham Paul Cochrane Ruslan Zakirov Sergey Romanov vagrant
+
+=over 4
+
+=item *
+
+Andrei Vereha <avereha@gmail.com>
+
+=item *
+
+Lee Johnson <lee@givengain.ch>
+
+=item *
+
+Michael Caron <michael.r.caron@gmail.com>
+
+=item *
+
+Michael Caron <mrcaron@users.noreply.github.com>
+
+=item *
+
+Nigel Metheringham <nm9762github@muesli.org.uk>
+
+=item *
+
+Paul Cochrane <paul@liekut.de>
+
+=item *
+
+Ruslan Zakirov <ruz@bestpractical.com>
+
+=item *
+
+Sergey Romanov <complefor@rambler.ru>
+
+=item *
+
+vagrant <vagrant@precise64.(none)>
+
+=back
+
 =cut
